﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace PrtOn
{
    internal partial class PBFont
    {
        /// <summary>
        /// 文字コード変換表
        /// </summary>
        Dictionary<string, int> codeTable = new Dictionary<string, int>
        {
            { " ",     0x00 },
            { "+",     0x01 },
            { "-",     0x02 },
            { "*",     0x03 },
            { "/",     0x04 },
            { "^",     0x05 },   // ↑
            { @"\UA",  0x05 },   // ↑
            { "!",     0x06 },
            { "\"",    0x07 },
            { "#",     0x08 },
            { "$",     0x09 },
            { ">",     0x0A },
            { @"\GE",  0x0B },   // ≧
            { "=",     0x0C },
            { @"\LE",  0x0D },   // ≦
            { "<",     0x0E },
            { @"\NE",  0x0F },   // ≠
            { "0",     0x10 },
            { "1",     0x11 },
            { "2",     0x12 },
            { "3",     0x13 },
            { "4",     0x14 },
            { "5",     0x15 },
            { "6",     0x16 },
            { "7",     0x17 },
            { "8",     0x18 },
            { "9",     0x19 },
            { ".",     0x1A },
            { @"\PI",  0x1B },   // π
            { ")",     0x1C },
            { "(",     0x1D },
            { @"\EM",  0x1E },   // E-
            { @"\EX",  0x1F },   // E
            { "A",     0x20 },
            { "B",     0x21 },
            { "C",     0x22 },
            { "D",     0x23 },
            { "E",     0x24 },
            { "F",     0x25 },
            { "G",     0x26 },
            { "H",     0x27 },
            { "I",     0x28 },
            { "J",     0x29 },
            { "K",     0x2A },
            { "L",     0x2B },
            { "M",     0x2C },
            { "N",     0x2D },
            { "O",     0x2E },
            { "P",     0x2F },
            { "Q",     0x30 },
            { "R",     0x31 },
            { "S",     0x32 },
            { "T",     0x33 },
            { "U",     0x34 },
            { "V",     0x35 },
            { "W",     0x36 },
            { "X",     0x37 },
            { "Y",     0x38 },
            { "Z",     0x39 },
            //{ @"\x3A", 0x3A },   // D
            //{ @"\x3B", 0x3B },   // L
            //{ @"\x3C", 0x3C },   // γ
            //{ @"\x3D", 0x3D },   // －に～
            //{ @"\x3E", 0x3E },   // σ
            //{ @"\x3F", 0x3F },   // -1
            { "a",     0x40 },
            { "b",     0x41 },
            { "c",     0x42 },
            { "d",     0x43 },
            { "e",     0x44 },
            { "f",     0x45 },
            { "g",     0x46 },
            { "h",     0x47 },
            { "i",     0x48 },
            { "j",     0x49 },
            { "k",     0x4A },
            { "l",     0x4B },
            { "m",     0x4C },
            { "n",     0x4D },
            { "o",     0x4E },
            { "p",     0x4F },
            { "q",     0x50 },
            { "r",     0x51 },
            { "s",     0x52 },
            { "t",     0x53 },
            { "u",     0x54 },
            { "v",     0x55 },
            { "w",     0x56 },
            { "x",     0x57 },
            { "y",     0x58 },
            { "z",     0x59 },
            //{ @"\x5A", 0x5A },   // “
            //{ @"\x5B", 0x5B },   // ”
            { "?",     0x5C },
            { ",",     0x5D },
            { ";",     0x5E },
            { ":",     0x5F },
            { @"\CI",  0x60 },   // 〇
            { @"\SG",  0x61 },   // Σ
            { @"\DG",  0x62 },   // ゜
            { @"\TR",  0x63 },   // △
            { "@",     0x64 },
            { @"\CR",  0x65 },   // ×
            { @"\DV",  0x66 },   // ÷
            { @"\SP",  0x67 },   // ♠
            { @"\S2",  0x67 },   // ♠
            { @"\LA",  0x68 },   // ←
            { @"\HT",  0x69 },   // ♥
            { @"\H2",  0x69 },   // ♥
            { @"\DI",  0x6A },   // ♦
            { @"\D2",  0x6A },   // ♦
            { @"\CL",  0x6B },   // ♣
            { @"\C2",  0x6B },   // ♣
            { @"\MU",  0x6C },   // μ
            { @"\OM",  0x6D },   // Ω
            { @"\DA",  0x6E },   // ↓
            { @"\RA",  0x6F },   // →
            { "%",     0x70 },
            //{ @"\",    0x71 },   // ￥
            { @"\SQ",  0x72 },   // □
            { "[",     0x73 },
            { "&",     0x74 },
            { "_",     0x75 },
            { "'",     0x76 },
            { @"\DT",  0x77 },   // ・
            { "]",     0x78 },
            { @"\BX",  0x79 },   // ■
            { @"\BS",  0x7A },   // ＼
            //{ @"\x7B", 0x7B },   // 🏁
            //{ @"\x7C", 0x7C },   // ∥
            //{ @"\x7D", 0x7D },   // b/t
            //{ @"\x7E", 0x7E },
            //{ @"\x7F", 0x7F },
            { @"\\",   0x71 }   // ￥
        };

        // 単純置換可能な特殊文字列
        static string[][] sptable1 = new string[][]
        {
            new string[] { ">=", @"\GE" },
            new string[] { "<=", @"\LE" },
            new string[] { "<>", @"\NE" },
            new string[] {@"\PI",@"PI"  },
            new string[] { "PI", @"\PI" }
        };

        // 単純置換不可能な特殊文字列
        // 直前に数字がある場合のみ変換する
        static string[][] sptable2 = new string[][]
        {
            new string[] { "EM", @"\EM" },  // E-
            new string[] { "EX", @"\EX" },  // E
        };


        /// <summary>
        /// PBの文字コード取得
        /// </summary>
        /// <param name="data">文字</param>
        /// <returns>
        /// int code  : PBの文字コード、変換できないときは0
        /// int length: 変換した文字数、変換できないときは1
        /// </returns>
        public (int code, int length) GetPBCode(string data)
        {
            foreach (var entry in codeTable)
            {
                if (data.StartsWith(entry.Key))
                {
                    return (entry.Value, entry.Key.Length);
                }
            }
            return (-1, 0);
        }

        /// <summary>
        /// 行から変換可能な文字列への変換
        /// </summary>
        /// <param name="input">行</param>
        /// <returns>変換後の行</returns>
        public string ConvertString(string input)
        {
            // "で囲まれた部分を一時的に保護する
            var quotedParts = new List<string>();
            input = MyRegex().Replace(input, match =>
            {
                quotedParts.Add(match.Value);
                return $"__QUOTE_{quotedParts.Count - 1}__";
            });

            // パターンを置換
            foreach (var replacement in sptable1)
            {
                input = Regex.Replace(input, Regex.Escape(replacement[0]), replacement[1]);
            }

            // パターンを置換
            string pattern = @"(?<=\d)";
            foreach (var replacement in sptable2)
            {
                input = Regex.Replace(input, pattern + replacement[0], replacement[1]);
            }

            // "で囲まれた部分を元に戻す
            for (int i = 0; i < quotedParts.Count; i++)
            {
                input = input.Replace($"__QUOTE_{i}__", quotedParts[i]);
            }


            return input;
        }

        [GeneratedRegex("\"([^\"]*)\"")]
        private static partial Regex MyRegex();
    }
}
